/**
 * OpenGL widget to display video as a texture on the 3D plane
 * Author: Andrey Nechypurenko (andreynech@gmail.com)
 */

#ifndef __GLWIDGET_H
#define __GLWIDGET_H

#include <qgl.h>


class GLWidget : public QGLWidget
{
  Q_OBJECT

 public:
  GLWidget(QWidget *parent = 0);
  virtual ~GLWidget();

  QSize minimumSizeHint() const;
  QSize sizeHint() const;

  void setupTex(unsigned int framew,
                unsigned int frameh,
                unsigned int texw,
                unsigned int texh,
                unsigned char *frame);

  void newFrame(unsigned char *frame);

 public slots:
  void setXRotation(int angle);
  void setYRotation(int angle);
  void setZRotation(int angle);

 signals:
  void xRotationChanged(int angle);
  void yRotationChanged(int angle);
  void zRotationChanged(int angle);

 protected:
  void initializeGL();
  void paintGL();
  void resizeGL(int width, int height);
  void mousePressEvent(QMouseEvent *event);
  void mouseMoveEvent(QMouseEvent *event);

 private:
  GLuint makeObject();
  void normalizeAngle(int *angle);

  GLuint object;
  int xRot;
  int yRot;
  int zRot;
  QPoint lastPos;
  QColor trolltechGreen;
  QColor trolltechPurple;

  GLuint texture[1];
  GLuint frameWidth;
  GLuint frameHeight;
  GLuint bufferWidth;
  GLuint bufferHeight;
};

#endif //__GLWIDGET_H
