/**
 * Author: Andrey Nechypurenko (andreynech@gmail.com)
 */

#include "Player.h"
#include <QMutexLocker>
#include <QMessageBox>


Player::Player()
  : capturer(&videoframe)
{
  this->controls.setupUi(this);
  this->glwidget = new GLWidget(this->controls.frmVideo);

  connect(this->controls.btnStopPlay, SIGNAL(clicked()), 
          this, SLOT(playStartStop()));
  connect(&(this->capturer), SIGNAL(setupTex()),
          this, SLOT(setupTex()), 
          Qt::QueuedConnection);
  connect(&(this->capturer), SIGNAL(newFrame()),
          this, SLOT(newFrame()), 
          Qt::QueuedConnection);
  connect(&(this->capturer), SIGNAL(error(QString)),
          this, SLOT(showErrMsg(QString)), 
          Qt::QueuedConnection);
}


void 
Player::resizeEvent(QResizeEvent *)
{
  QSize s = this->controls.frmVideo->size();
  s.setWidth(s.width() - 8);
  s.setHeight(s.height() - 8);
  this->glwidget->move(4, 4);
  this->glwidget->resize(s);
}


void 
Player::playStartStop()
{
  if(this->capturer.isRunning())
    {
      this->capturer.shouldStop();
      this->capturer.wait();
      this->controls.btnStopPlay->setIcon(QIcon(QString::fromUtf8(":/images/images/play.xpm")));
    }
  else
    {
      this->capturer.start();
      this->controls.btnStopPlay->setIcon(QIcon(QString::fromUtf8(":/images/images/stop.xpm")));
    }
}


void 
Player::setupTex()
{
  QMutexLocker lock(&(this->videoframe.mutex));
  this->glwidget->setupTex(this->videoframe.width,
                           this->videoframe.height,
                           this->videoframe.texwidth,
                           this->videoframe.texheight,
                           this->videoframe.texdata);         
}


void 
Player::newFrame()
{
  QMutexLocker lock(&(this->videoframe.mutex));
  this->glwidget->newFrame(this->videoframe.texdata);
}


void 
Player::showErrMsg(QString msg)
{
  QMessageBox msgBox(QMessageBox::Warning,
                     tr("Runtime error"),
                     msg,
                     QMessageBox::Ok,
                     this);
  msgBox.exec();
}
