#ifndef SRAYMID_H_
#define SRAYMID_H_
//Copyright Sensoray Company Inc. 2007


#ifndef DLL_BUILD
typedef unsigned int UINT32;
typedef int SINT32;
typedef char SINT8;
typedef unsigned char UINT8;
#endif

#define MAX_BOARDS 8
// error constants 
#define MIDSRAY_SUCCESS             0
#define MIDSRAY_ERR_NONE            0
#define MIDSRAY_ERR_UNSPECIFIED     -1
#define MIDSRAY_ERR_CAPTURING       -2   // already capturing/recording
#define MIDSRAY_ERR_NOTCAPTURING    -3   // not capturing/recording
#define MIDSRAY_ERR_RUNNING         -4   // attempt to set configuration while graph running
#define MIDSRAY_ERR_SNAPSHOT        -5   // currently displaying snapshot
#define MIDSRAY_ERR_INVALIDFILE     -6   // invalid file
#define MIDSRAY_ERR_DIRECTX         -7   // directX error( other specific directX errors are possible)
#define MIDSRAY_ERR_NODEVICE		-8   // device not present
#define MIDSRAY_ERR_NOTINITIALIZED  -9	 // initialization function failed or not called
#define MIDSRAY_ERR_INVALIDMODE    -10  // invalid mode(eg.  record in MJPEG CAPTURE mode)
#define MIDSRAY_ERR_NOMEM          -11  // no memory
#define MIDSRAY_ERR_INVALIDPARAM   -12
#define MIDSRAY_ERR_INVALIDUSERDATA -13  // user data contains MPEG start code 
                                         // user data must not have more than 23 consecutive bits that are zero
                                         // The API user should insert marker codes to avoid this if necessary.
#define MIDSRAY_ERR_NONEWDATA       -14  // no new data


/** video systems.  PAL or NTSC
 */
typedef enum
{
	MIDSRAY_VIDSYS_PAL=1,
	MIDSRAY_VIDSYS_NTSC=2
} MIDSRAY_VIDSYS;


typedef enum
{
	MIDSRAY_ENCODE_MPEG1 = 0,
	MIDSRAY_ENCODE_MPEG2 = 1,
	MIDSRAY_ENCODE_MPEG4 = 2,  // MP4S
	MIDSRAY_ENCODE_MOTIONJPEG = 3,
    MIDSRAY_ENCODE_MPEG4DIVX = 9, 
} MIDSRAY_ENCODING;


#define MAX_DATASIZE     104


#define MACROBLOCK_MAP_SIZE     ((1620/16)+3)
typedef struct
{
    int data_length;
    int type; //0 Iframe, 1 P frame, 2 Bframe
    struct timeval timestamp;
} frame_info_t;

#ifdef __cplusplus
extern "C"
{
#endif

// open SRAY board
int SN_OpenBoard( int boardnum);
// close SRAY board
int SN_CloseBoard( int boardnum);

// set the basic chip settings
// Note: settings are not updated until stream started(ie. if board running, must stop and restart to take effect).
// vsys determines the video size.  
// If call SN_SetVideoSystem, you may want to call this function also to update the resolution. 
// @param boardnum board number
// @param encodetype MPEG1, MPEG2, MPEG2 or MOTIONJPEG
// @param iVidSize iVidSize == 0(full size), iVidSize ==1 (half size)
// @param vsys video system(to determine resolution sizes 720x480(NTSC full) or 720x576(PAL full size)
// @param bitrate desired bitrate (default 2Mbit/s).  For iVidSize=1(half size), max bitrate is 2MBit/s.
// @param bIFramesOnly (default off=0).  To use IFrames only in the stream.
// SN_SetSeqMode overrides this setting    
int SN_SetBasicChipSettings( int boardnum, MIDSRAY_ENCODING encodetype, int iVidSize, MIDSRAY_VIDSYS vsys, int bitrate, int bIFramesOnly );


// start video and audio streams
int SN_StartStream(int boardnum);
// stop streams
int SN_StopStream(int boardnum);

// get one frame of video data in pBuf.  BufLen specified in BufLen. size of frame returns in pFI->VideoLength
// returns 0 on success
int SN_GetOneFrame(int boardnum, unsigned char *pBuf, int BufLen, frame_info_t *pFI);

// get audio samples in pBuf.  Set buffer size in BufLen.  Returns number of samples in buflen
// returns 0 on success
int SN_GetAudioData( int boardnum, unsigned char *pBuf, int *BufLen);


// set the video source 0 -- svideo, 1 -- composite(2250 only)
// 314( 
int SN_SetVideoSource( int boardnum, int source);

// set video system.  0--NTSC, 1 PAL
int SN_SetVideoSystem( int boardnum, MIDSRAY_VIDSYS vidsys);

// levels bright 0-100 (default 50)
int SN_SetBrightness( int boardnum, int bright);
// contrast 0-100 (default 50)
int SN_SetContrast( int boardnum, int contrast);
// hue -50 to 50 (default 0)
int SN_SetHue( int boardnum, int hue);
// saturation 0-100 ( default 50)
int SN_SetSaturation( int boardnum, int sat);
// set caption text
int SN_SetCaption( int boardnum, int xstart, int ystart, char *text);
// clear caption text
int SN_ClearCaption( int boardnum);



//disable AVSync in driver.  arg=1 AVSYNC disable, arg=0 AVSYNC enabled
// recommend turning off AVSync after startstream with MOTIONJPEG encoding
int SN_DisableAVSync( int boardnum, UINT32 arg);

// source = 0(default line), source=1(mic), source =2 (mic with boost)
int SN_SetAudioSource( int boardnum, int source);

// set FPS on the fly( after SN_StartStream)
// allowed values NTSC
// 1,2,3,4,5,6,7,10,15,30(full)
// allowed values PAL
// 1,2,3,4,5,6,8,12,25
int SN_SetFPS_OTF( int boardnum, int fps);


// set FPS before starting stream
// allowed values NTSC
// 1,2,3,4,5,6,7,10,15,30(full)
// allowed values PAL
// 1,2,3,4,5,6,8,12,25
int SN_SetFPS( int boardnum, int fps);
    

int SN_GetFPS( int boardnum, int *fps);


int SN_SetBitrate( int boardnum, int bitrate);
int SN_GetBitrate( int boardnum, int *bitrate);


int SN_SetHeight( int boardnum, int height);
int SN_GetHeight( int boardnum, int *height);

int SN_SetWidth( int boardnum, int w);
int SN_GetWidth( int boardnum, int *w);


int SN_GetVidSys( int boardnum, MIDSRAY_VIDSYS *pvsys);

// advanced used only. GOPs must be valid
int SN_SetGopSize( int boardnum, int size);
int SN_GetGopSize( int boardnum, int *size);
// advanced use only.  call after SN_SetBasicChipSettings
// emode = 1 (Iframes only), 2(IP), 3(IPB)    
int SN_SetSeqMode( int boardnum, int eMode);
int SN_GetSeqMode( int boardnum, int *eMode);

int SN_GetEncodeType( int boardnum, MIDSRAY_ENCODING *encode);
#ifdef __cplusplus
}
#endif


#endif


